/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/*!
 * \file src/relax/transform/gradient_simplifier.h
 * \brief Simplify patterns generated by the gradient pass. Especially, simplify the matmul
 * patterns.
 * \sa tvm/relax/transform/gradient.cc
 */

#ifndef TVM_RELAX_TRANSFORM_GRADIENT_SIMPLIFIER_H_
#define TVM_RELAX_TRANSFORM_GRADIENT_SIMPLIFIER_H_

#include <tvm/relax/expr.h>

namespace tvm {
namespace relax {

/*!
 * \brief Simplify patterns generated by the gradient pass. Especially, simplify the matmul
 * patterns.
 * \param func The function to be simplified.
 * \return The simplified function.
 */
Function SimplifyGradient(const Function& func);

}  // namespace relax
}  // namespace tvm

#endif  // TVM_RELAX_TRANSFORM_GRADIENT_SIMPLIFIER_H_
